/******************************************************************************/
/*                                                                            */
/*                           P C I - B I O S   V1.13                          */
/*                                                                            */
/*                        Das PCI-Bios fr HADES040/060                       */
/*                                                                            */
/*  Modul:         PCI-BIOS (PCI_BIOS.LIB)                                    */
/*  File:          PCI_LIB.C                                                  */
/*  Teilaufgabe:   PCI-Library                                                */
/*  Beschreibung:  Installationsroutine fr das PCI-BIOS Interface            */
/*  Bemerkung:     berprft, ob ein _PCI Cookie installiert ist, und ob die  */
/*                 Versionsnummer des Routinenpools auch mit der Library      */
/*                 untersttzt wird. Anderenfalls wird eine entsprechende     */
/*                 Fehlermeldung an das aufrufende Programm zurckgegeben.    */
/*  Funktionen:    pcibios_init                                               */
/*                                                                            */
/*  (c)1997-2000 Markus Fichtenbauer                             03.08.2000   */
/*                                                                            */
/******************************************************************************/
/* $Id$ */
/******************************************************************************/
/* $Log$ */

#include <portab.h>                   /* Portab-Deklarationen (ULONG,...)     */
#include <stddef.h>                   /* Standard-Definitionen (NULL,...)     */
#include <tos.h>                      /* Betriebsystemfunktionen (Super,...)  */
#include <pci_bios.h>                 /* PCI-BIOS Interface                   */
#include <pci.h>                      /* allgemeine PCI Deklarationen         */
#include "pci_int.h"                  /* PCI-BIOS interne Deklarationen       */

/******************************************************************************/
/*                                                                            */
/*                External Functions / Local Function Prototypes              */
/*                                                                            */
/******************************************************************************/
LONG pcibios_init(void);              /* Initialisierung des BIOS-Interface   */
LONG read_pcicookie (void);           /* PCI-Cookie lesen                     */

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  LONG pcibios_init (void)                                  */
/*                                                                            */
/*  Beschreibung:   sucht nach einem _PCI Cookie und stellt die Versions-     */
/*                  nummer des Routinenpools fest.                            */
/*                                                                            */
/*  Parameter:      Returnwert = PCI-BIOS Fehlercode                          */
/*                                                                            */
/******************************************************************************/
LONG pcibios_init (void)
{
  PCI_COOKIE *bios_cookie = NULL;      /* Pointer auf PCI-BIOS Struktur       */
  LONG fehler = PCI_SUCCESSFUL;        /* Fehlercode                          */

/* PCI-BIOS Cookie lesen                                                      */
   bios_cookie = (PCI_COOKIE *)(Supexec(read_pcicookie));

/* Wenn der Cookie vorhanden ist                                              */
  if (bios_cookie != NULL)
  {
/*  Um welche Version des Routinenpools handelt es sich denn berhaupt ?      */
    switch (HIGHW(bios_cookie->version))
    {
      case (0x0001):                   /* aktuelle Version                    */
        (LONG)find_pci_device           = (LONG)(bios_cookie->routine[0]);
        (LONG)find_pci_classcode        = (LONG)(bios_cookie->routine[1]);
        (LONG)read_config_byte          = (LONG)(bios_cookie->routine[2]);
        (LONG)read_config_word          = (LONG)(bios_cookie->routine[3]);
        (LONG)read_config_longword      = (LONG)(bios_cookie->routine[4]);
        (LONG)fast_read_config_byte     = (LONG)(bios_cookie->routine[5]);
        (LONG)fast_read_config_word     = (LONG)(bios_cookie->routine[6]);
        (LONG)fast_read_config_longword = (LONG)(bios_cookie->routine[7]);
        (LONG)write_config_byte         = (LONG)(bios_cookie->routine[8]);
        (LONG)write_config_word         = (LONG)(bios_cookie->routine[9]);
        (LONG)write_config_longword     = (LONG)(bios_cookie->routine[10]);
        (LONG)hook_interrupt            = (LONG)(bios_cookie->routine[11]);
        (LONG)unhook_interrupt          = (LONG)(bios_cookie->routine[12]);
        (LONG)special_cycle             = (LONG)(bios_cookie->routine[13]);
        (LONG)get_routing               = (LONG)(bios_cookie->routine[14]);
        (LONG)set_interrupt             = (LONG)(bios_cookie->routine[15]);
        (LONG)get_resource              = (LONG)(bios_cookie->routine[16]);
        (LONG)get_card_used             = (LONG)(bios_cookie->routine[17]);
        (LONG)set_card_used             = (LONG)(bios_cookie->routine[18]);
        (LONG)read_mem_byte             = (LONG)(bios_cookie->routine[19]);
        (LONG)read_mem_word             = (LONG)(bios_cookie->routine[20]);
        (LONG)read_mem_longword         = (LONG)(bios_cookie->routine[21]);
        (LONG)fast_read_mem_byte        = (LONG)(bios_cookie->routine[22]);
        (LONG)fast_read_mem_word        = (LONG)(bios_cookie->routine[23]);
        (LONG)fast_read_mem_longword    = (LONG)(bios_cookie->routine[24]);
        (LONG)write_mem_byte            = (LONG)(bios_cookie->routine[25]);
        (LONG)write_mem_word            = (LONG)(bios_cookie->routine[26]);
        (LONG)write_mem_longword        = (LONG)(bios_cookie->routine[27]);
        (LONG)read_io_byte              = (LONG)(bios_cookie->routine[28]);
        (LONG)read_io_word              = (LONG)(bios_cookie->routine[29]);
        (LONG)read_io_longword          = (LONG)(bios_cookie->routine[30]);
        (LONG)fast_read_io_byte         = (LONG)(bios_cookie->routine[31]);
        (LONG)fast_read_io_word         = (LONG)(bios_cookie->routine[32]);
        (LONG)fast_read_io_longword     = (LONG)(bios_cookie->routine[33]);
        (LONG)write_io_byte             = (LONG)(bios_cookie->routine[34]);
        (LONG)write_io_word             = (LONG)(bios_cookie->routine[35]);
        (LONG)write_io_longword         = (LONG)(bios_cookie->routine[36]);
        (LONG)get_machine_id            = (LONG)(bios_cookie->routine[37]);
        (LONG)get_pagesize              = (LONG)(bios_cookie->routine[38]);
        (LONG)virt_to_bus               = (LONG)(bios_cookie->routine[39]);
        (LONG)bus_to_virt               = (LONG)(bios_cookie->routine[40]);
        (LONG)virt_to_phys              = (LONG)(bios_cookie->routine[41]);
        (LONG)phys_to_virt              = (LONG)(bios_cookie->routine[42]);
      break;

/*    hier folgen dann die lteren Versionen des Routinenpools                */

      case 0:                          /* da ist nur PCI_CONF installiert     */
        fehler = PCI_BIOS_NOT_INSTALLED;
      break;

      default:                         /* alte oder unbekannte Pool-Version   */
/*      das muss dann in den nchsten Versionen auscodiert werden, vorlufig  */
/*      gibt's ja nur eine einzige Pool-Version.                              */
        fehler = PCI_BIOS_WRONG_VERSION;
    }
  }
  else
    fehler = PCI_BIOS_NOT_INSTALLED;   /* kein _PCI Cookie vorhanden          */

  return fehler;
}

LONG read_pcicookie()
{
  COOKIE_ENTRY *cookie;

  if (*(LONG **)_p_cookies == NULL) return (0L);

  for (cookie = (COOKIE_ENTRY *)(*(LONG **)_p_cookies); cookie->magic != 0L; cookie++)
  {
    if (cookie->magic == COOKIE_PCI)
      return (cookie->value);
  }

  return (0L);
}
